@@ -18,7 +18,6 @@ import cv2
 import numpy as np
 import numpy.typing as npt
 import skimage.io as io
-import tifffile
 import skimage.transform as trans
 from skimage.measure import label
 import skimage.morphology as morph
@@ -99,10 +98,8 @@ def readreshape(
 
     """
     fext = os.path.splitext(filename)[1].lower()
-    if fext in (".png", ".jpg", ".jpeg"):
-        i = io.imread(filename, as_gray=True)
-    elif fext in (".tif", ".tiff"):
-        i = tifffile.imread(filename)
+    if fext in (".png", ".jpg", ".jpeg", ".tif", ".tiff"):
+        i = cv2.imread(filename, cv2.IMREAD_ANYDEPTH)
     else:
         raise ValueError("Only PNG, JPG or single-page TIF files accepted")
     if i.ndim == 3:
@@ -910,7 +907,7 @@ def seg_weights_2D(
     mask: utils.SegmentationMask, classweights: Tuple[int, int] = (1, 1)
 ) -> npt.NDArray[np.float32]:
     """
-    Compute custom weightmaps designed for bacterial images 
+    Compute custom weightmaps designed for bacterial images
     where borders are difficult to distinguish
 
     Parameters
@@ -1277,7 +1274,7 @@ def getfnfromprototype(
 def predictCompilefromseg_track(
     img_path: str,
     seg_path: str,
-    files_list: Union[List[str], Tuple[str, ...]] = [],
+    files_list: Union[List[str], Tuple[str, ...]],
     target_size: Tuple[int, int] = (256, 32),
     crop: bool = False,
 ) -> Tuple[
@@ -1299,8 +1296,6 @@ def predictCompilefromseg_track(
         same as in the img_path folder.
     files_list : tuple/list of strings, optional
         List of filenames to compile in the img_path and seg_path folders.
-        If empty, all files in the folder will be read.
-        The default is [].
     target_size : tuple of 2 ints, optional
         Input and output image size.
         The default is (256,32).
@@ -1325,16 +1320,14 @@ def predictCompilefromseg_track(
     """
 
     seg_name_list = []
-    if not files_list:
-        files_list = sorted(os.listdir(img_path))
 
     ind = 0
 
     numstrs = re.findall(
-        r"\d+", files_list[0]
+        r"\d+", os.path.basename(files_list[0])
     )  # Get digits sequences in first filename
     charstrs = re.findall(
-        r"\D+", files_list[0]
+        r"\D+", os.path.basename(files_list[0])
     )  # Get character sequences in first filename
 
     # Create the string prototype to be used to generate filenames on the fly:
@@ -1349,38 +1342,25 @@ def predictCompilefromseg_track(
         )
     elif len(numstrs) == 3 and len(charstrs) == 4:
         # Order is position, chamber, frame/timepoint
-        prototype = (
-            charstrs[0]
-            + "%0"
-            + str(len(numstrs[0]))
-            + "d"
-            + charstrs[1]
-            + "%0"
-            + str(len(numstrs[1]))
-            + "d"
-            + charstrs[2]
-            + "%0"
-            + str(len(numstrs[2]))
-            + "d"
-            + charstrs[3]
+        protoype = (
+            f"{charstrs[0]}%0{len(numstrs[1])}d"
+            f"{charstrs[1]}%0{len(numstrs[2])}d"
+            f"{charstrs[2]}%0{len(numstrs[3])}d"
+            f"{charstrs[3]}"
         )
         fileorder = "pct"
     elif len(numstrs) == 2 and len(charstrs) == 3:
         # Order is position, frame/timepoint
-        prototype = (
-            charstrs[0]
-            + "%0"
-            + str(len(numstrs[0]))
-            + "d"
-            + charstrs[1]
-            + "%0"
-            + str(len(numstrs[1]))
-            + "d"
-            + charstrs[2]
+        protoype = (
+            f"{charstrs[0]}%0{len(numstrs[1])}d"
+            f"{charstrs[1]}%0{len(numstrs[2])}d"
+            f"{charstrs[3]}"
         )
         fileorder = "pt"
         crop = True
     fileordercell = fileorder + "n"
+
+    boxes = []
     for index, item in enumerate(files_list):
         filename = os.path.basename(item)
         # Get position, chamber & frame numbers:
@@ -1429,7 +1409,6 @@ def predictCompilefromseg_track(
             lblimg, lblnb = label(prevsegall, connectivity=1, return_num=True)
 
             x = np.zeros(shape=(lblnb,) + target_size + (4,), dtype=np.float32)
-            boxes = []
 
             for lbl in range(1, lblnb + 1):
                 seg = lblimg == lbl
@@ -1567,7 +1546,7 @@ def saveResult_track(
         if files_list:
             filenameMo = os.path.join(save_path, os.path.basename(files_list[i]))
         else:
-            filenameMo = os.path.join(save_path, "tracking_%09d.png" % i)
+            filenameMo = os.path.join(save_path, f"tracking_{i:09d}.png")
         with warnings.catch_warnings():
             warnings.simplefilter("ignore")
             io.imsave(filenameMo, (mother * 255).astype(np.uint8))
